# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

r"""Creates versioned files and information files of the files.

  When the version is "1.2.3.4", the following command creates these four files.
    /PATH/TO/file1-1.2.3.4.ext      : Copied from /PATH/TO/file1.ext
    /PATH/TO/file1-1.2.3.4.info.ext : File information about file1.ext
    /PATH/TO/file2-1.2.3.4          : Copied from /PATH/TO/file2
    /PATH/TO/file2-1.2.3.4.info     : File information about file2

  % python versioning_files.py --version_file mozc_version.txt \
      --configuration Release /PATH/TO/file1.ext /PATH/TO/file2

  If configuration is "Debug" these file names will be
    "/PATH/TO/file1-Debug-1.2.3.4.ext"
    "/PATH/TO/file1-Debug-1.2.3.4.info.ext"
    "/PATH/TO/file2-Debug-1.2.3.4"
    "/PATH/TO/file2-Debug-1.2.3.4.info"
"""

__author__ = "horo"

import base64
import hashlib
import logging
import optparse
import os
import shutil
import six

from build_tools import mozc_version


def _GetSha1Digest(file_path):
  """Returns the sha1 hash of the file."""
  sha = hashlib.sha1()
  with open(file_path, 'rb') as f:
    data = f.read()
    sha.update(data)
  return sha.digest()


def _GetBuildId():
  """Returns the build ID or an empty string."""
  envvar_list = []

  for envvar in envvar_list:
    # If the value is empty, it should be skipped.
    if os.environ.get(envvar):
      return os.environ[envvar]
  return ''


def _VersioningFile(version_string, is_debug, file_path):
  """Creates a versioned file and a information file of the file."""
  build_id = _GetBuildId()
  file_path_base, ext = os.path.splitext(file_path)
  if is_debug:
    file_path_base += '-Debug'
  new_file_path = '%s-%s%s' % (file_path_base, version_string, ext)
  shutil.copyfile(file_path, new_file_path)
  package = os.path.basename(new_file_path)
  file_size = os.path.getsize(new_file_path)
  sha1_digest = _GetSha1Digest(new_file_path)
  if six.PY3:
    sha1_hash = base64.b64encode(sha1_digest).decode('latin1')
    sha1_hash_hex = sha1_digest.hex()
  else:
    sha1_hash = base64.b64encode(sha1_digest)
    sha1_hash_hex = sha1_digest.encode('hex')
  with open('%s.info' % new_file_path, 'w') as output:
    output.write('package\t%s\n' % package)
    output.write('build_id\t%s\n' % build_id)
    output.write('version\t%s\n' % version_string)
    output.write('size\t%s\n' % file_size)
    output.write('sha1_hash\t%s\n' % sha1_hash)
    output.write('sha1_hash_hex\t%s\n' % sha1_hash_hex)


def VersioningFiles(version_string, is_debug, file_paths):
  """Creates versioned files and information files of the files."""
  for file_path in file_paths:
    _VersioningFile(version_string, is_debug, file_path)


def main():
  """The main function."""
  parser = optparse.OptionParser()
  parser.add_option('--version_file', dest='version_file')
  parser.add_option('--configuration', dest='configuration')

  (options, args) = parser.parse_args()
  if options.version_file is None:
    logging.error('--version_file is not specified.')
    exit(-1)
  if options.configuration is None:
    logging.error('--configuration is not specified.')
    exit(-1)
  version = mozc_version.MozcVersion(options.version_file)
  version_string = version.GetVersionString()
  is_debug = (options.configuration == 'Debug')
  VersioningFiles(version_string, is_debug, args)

if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""A checker tool to make sure if the binary size is too huge.

Usage:
  % python binary_size_checker.py --target_directory=out_linux/Debug
"""

from __future__ import absolute_import
from __future__ import print_function

import logging
import optparse
import os
import sys


# The expected maximum size of each package and binary.  Each size is
# in Megabytes.
EXPECTED_MAXIMUM_SIZES = {
    # Distribution package files:
    'GoogleJapaneseInput.dmg': 70,
    'GoogleJapaneseInput32.msi': 65,
    'GoogleJapaneseInput64.msi': 65,
    }


def CheckFileSize(filename):
  """Check the size of filename.

  Args:
    filename: the file name to be scanned.

  Returns:
    True is returned when filename is not target or does not exceed the limit.
  """
  basename = os.path.basename(filename)
  if basename not in EXPECTED_MAXIMUM_SIZES:
    return True

  actual_size = os.stat(filename).st_size
  expected_size = EXPECTED_MAXIMUM_SIZES[basename]
  if actual_size < expected_size * 1024 * 1024:
    print('Pass: %s (size: %d) is smaller than expected (%d MB)' % (
        filename, actual_size, expected_size))
    return True
  else:
    print('WARNING: %s (size: %d) is larger than expected (%d MB)' % (
        filename, actual_size, expected_size))
    return False


def ScanAndCheckSize(directory):
  """Scan the specified directory and check the binary/package size.

  Args:
    directory: the directory file name to be scanned.

  Returns:
    True is returned when all criteria is passed.
  """
  result = True
  for filename in os.listdir(directory):
    result &= CheckFileSize(os.path.join(directory, filename))
  return result


def ParseOptions():
  """Parse command line options.

  Returns:
    The options data passed.
  """
  parser = optparse.OptionParser()
  parser.add_option('--target_directory', dest='target_directory',
                    help='The directory which will contain target binaries '
                    'and packages.')
  parser.add_option('--target_filename', dest='target_filename',
                    help='The target filenames (comma separated).')
  (options, unused_args) = parser.parse_args()
  return options


def main():
  """The main function."""
  options = ParseOptions()
  if not (options.target_directory or options.target_filename):
    logging.error('--target_directory or --target_filename is not specified.')
    sys.exit(-1)

  if options.target_directory:
    if not ScanAndCheckSize(options.target_directory):
      logging.error('Files in target_directory exceeds the limit.')
      sys.exit(-1)
  if options.target_filename:
    for filename in options.target_filename.split(','):
      if not CheckFileSize(filename):
        logging.error('The target_filename exceeds the limit.')
        sys.exit(-1)


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""A tool to generate boundary data.

Bounday data binary image is an array of uint16 whose length is 2N, where N is
the number of POS IDs including special POS.  The array has the following
structure:

-------------------------------------
prefix penalty of POS ID 0 (2 bytes)
-------------------------------------
suffix penalty of POS ID 0 (2 bytes)
-------------------------------------
prefix penalty of POS ID 1 (2 bytes)
-------------------------------------
suffix penalty of POS ID 1 (2 bytes)
-------------------------------------
  .
  .
  .
-------------------------------------
prefix penalty of POS ID N (2 bytes)
-------------------------------------
suffix penalty of POS ID N (2 bytes)
-------------------------------------

See converter/segmenter.cc for how it's used.
"""

from __future__ import absolute_import
from __future__ import print_function

import codecs
import optparse
import re
import struct
import sys

from six.moves import range


def PatternToRegexp(pattern):
  return '^' + pattern.replace('*', '[^,]+')


def LoadPatterns(file):
  prefix = []
  suffix = []
  for line in codecs.open(file, 'r', encoding='utf-8'):
    if len(line) <= 1 or line[0] == '#':
      continue
    fields = line.split()
    label = fields[0]
    feature = fields[1]
    cost = int(fields[2])
    if cost < 0 or cost > 0xffff:
      sys.exit(-1)
    if label == 'PREFIX':
      prefix.append([re.compile(PatternToRegexp(feature)), cost])
    elif label == 'SUFFIX':
      suffix.append([re.compile(PatternToRegexp(feature)), cost])
    else:
      print('format error %s' % (line))
      sys.exit(0)
  return (prefix, suffix)


def GetCost(patterns, feature):
  for p in patterns:
    pat = p[0]
    cost = p[1]
    if pat.match(feature):
      return cost
  return 0


def LoadFeatures(filename):
  features = []
  for line in codecs.open(filename, 'r', encoding='utf-8'):
    fields = line.split()
    features.append(fields[1])
  return features


def CountSpecialPos(filename):
  count = 0
  for line in codecs.open(filename, 'r', encoding='utf-8'):
    line = line.rstrip()
    if not line or line[0] == '#':
      continue
    count += 1
  return count


def ParseOptions():
  parser = optparse.OptionParser()
  parser.add_option('--boundary_def', dest='boundary_def',
                    help='Boundary definition file')
  parser.add_option('--id_def', dest='id_def',
                    help='Boundary definition file')
  parser.add_option('--special_pos', dest='special_pos',
                    help='Special POS definition file')
  parser.add_option('--output', dest='output',
                    help='Output binary file')
  return parser.parse_args()[0]


def main():
  opts = ParseOptions()

  prefix, suffix = LoadPatterns(opts.boundary_def)
  features = LoadFeatures(opts.id_def)
  num_special_pos = CountSpecialPos(opts.special_pos)

  with open(opts.output, 'wb') as f:
    for feature in features:
      f.write(struct.pack('<H', GetCost(prefix, feature)))
      f.write(struct.pack('<H', GetCost(suffix, feature)))

    for _ in range(num_special_pos):
      f.write(struct.pack('<H', 0))
      f.write(struct.pack('<H', 0))


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

__author__ = "taku"

import codecs
import optparse


def IsPrefix(s, key):
  try:
    n = s.index(key)
  except:
    n = -1

  if n == 0:
    return True
  else:
    return False


def LoadRewriteMapRule(filename):
  fh = codecs.open(filename, encoding='utf-8')
  rule = []
  for line in fh:
    line = line.rstrip('\n')
    if not line or line.startswith('#'):
      continue
    fields = line.split()
    rule.append([fields[0], fields[1]])
  return rule


def ReadPOSID(id_file, special_pos_file):
  pos_list = []

  for line in codecs.open(id_file, 'r', encoding='utf-8'):
    fields = line.split()
    pos_list.append(fields[1])

  for line in codecs.open(special_pos_file, 'r', encoding='utf-8'):
    if len(line) <= 1 or line[0] == '#':
      continue
    fields = line.split()
    pos_list.append(fields[0])

  return pos_list


def ParseOptions():
  parser = optparse.OptionParser()
  parser.add_option('--id_def', dest='id_def',
                    help='POS ID definition file')
  parser.add_option('--special_pos', dest='special_pos',
                    help='Special POS definition file')
  parser.add_option('--pos_group_def', dest='pos_group_def',
                    help='Left POS ID group definition file')
  parser.add_option('--output', dest='output',
                    help='Output file for binary mode')
  return parser.parse_args()[0]


def main():
  opts = ParseOptions()

  # read lid file
  pos_list = ReadPOSID(opts.id_def, opts.special_pos)

  # read rule file
  rules = LoadRewriteMapRule(opts.pos_group_def)

  current_id = 1
  id_map = {}
  ids = bytearray()

  for target in pos_list:
    id = 0
    for rule in rules:
      if IsPrefix(target, rule[0]):
        if rule[1] in id_map:
          id = id_map[rule[1]]
        else:
          id = current_id
          id_map[rule[1]] = current_id
          current_id += 1
    ids.append(id)

  with open(opts.output, 'wb') as f:
    f.write(ids)


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Library replacing targets formmated ${like_this} with specified variables.
"""

__author__ = "komatsu"

import re


def ReplaceVariables(data, environment):
  """Replace all occurrence of the variable in data by the value.

  Args:
    data: the original data string
    environment: a dict which maps from variable names to their values

  Returns:
    the data string which replaces the variables by the value.
  """
  def Replacer(matchobj):
    """The replace function to expand variables.

    Args:
      matchobj: the match object

    Returns:
      The value for the variable if the variable is defined in the
      environment.  Otherwise original string is returned.
    """
    if matchobj.group(1) in environment:
      return environment[matchobj.group(1)]
    return matchobj.group(0)

  return re.sub(r'\$\{(\w+)\}', Replacer, data)



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Generate zero query data file for number suffixes."""

import codecs
import collections
import optparse

from prediction import gen_zero_query_util as util


def ReadZeroQueryNumberData(input_stream):
  """Reads zero query number data from stream and returns zero query data."""
  zero_query_dict = collections.defaultdict(list)

  for line in input_stream:
    if line.startswith('#'):
      continue
    line = line.rstrip('\r\n')
    if not line:
      continue

    tokens = line.split('\t')
    key = tokens[0]
    values = tokens[1].split(',')

    for value in values:
      zero_query_dict[key].append(
          util.ZeroQueryEntry(util.ZERO_QUERY_TYPE_NUMBER_SUFFIX,
                              value, util.EMOJI_TYPE_NONE, 0))
  return zero_query_dict


def ParseOption():
  """Parses command line options."""
  parser = optparse.OptionParser()
  parser.add_option('--input', dest='input', help='Input file path')
  parser.add_option('--output_token_array', dest='output_token_array',
                    help='Output token array file path')
  parser.add_option('--output_string_array', dest='output_string_array',
                    help='Output string array file path')
  return parser.parse_args()[0]


def main():
  options = ParseOption()
  with codecs.open(options.input, 'r', encoding='utf-8') as input_stream:
    zero_query_dict = ReadZeroQueryNumberData(input_stream)
  util.WriteZeroQueryData(zero_query_dict,
                          options.output_token_array,
                          options.output_string_array)


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Generate a C++ array definition for file embedding."""

from __future__ import absolute_import
from __future__ import print_function
import binascii
import optparse
import os
import six
from six.moves import range


def _ParseOption():
  """Parse command line options."""
  parser = optparse.OptionParser()
  parser.add_option('--input', dest='input')
  parser.add_option('--name', dest='name')
  parser.add_option('--output', dest='output')
  return parser.parse_args()[0]


def _FormatAsUint64LittleEndian(s):
  """Formats a string as uint64 value in little endian order."""
  for _ in range(len(s), 8):
    s += six.b('\0')
  s = s[::-1]  # Reverse the string
  return six.b('0x%s') % binascii.b2a_hex(s)


def main():
  opts = _ParseOption()
  with open(opts.input, 'rb') as infile:
    with open(opts.output, 'wb') as outfile:
      outfile.write(six.b(
          '#ifdef MOZC_EMBEDDED_FILE_%(name)s\n'
          '#error "%(name)s was already included or defined elsewhere"\n'
          '#else\n'
          '#define MOZC_EMBEDDED_FILE_%(name)s\n'
          'const uint64 %(name)s_data[] = {\n'
          % {'name': opts.name}))

      while True:
        chunk = infile.read(8)
        if not chunk:
          break
        outfile.write(six.b('  '))
        outfile.write(_FormatAsUint64LittleEndian(chunk))
        outfile.write(six.b(',\n'))

      outfile.write(six.b(
          '};\n'
          'const EmbeddedFile %(name)s = {\n'
          '  %(name)s_data,\n'
          '  %(size)d,\n'
          '};\n'
          '#endif  // MOZC_EMBEDDED_FILE_%(name)s\n'
          % {'name': opts.name,
             'size': os.stat(opts.input).st_size}))


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Provides parallelization tests.

Usage:

The usage example is as follows.

 Sample Code:
  launcher = TestLauncher(gtest_report_dir='/tmp/my_gtest_report_dir')
  launcher.AddTest(['/path/to/binary/some_test', '--logtostderr' ])
  launcher.AddTest(['/path/to/binary/another_test', '--logtostderr' ])
  ...
  launcher.Execute(8)          # Execute with specified number of processes.
                                 This function blocks main thread until all
                                 task is completed.
TODO(nona): Adds unit test(hard to inject mock code due to multiprocessing)
"""

__author__ = "nona"

import errno
import logging
import multiprocessing
import os
import shutil
import stat
import subprocess
import sys
import tempfile
import time

from build_tools import util


# TODO(team): Move this to build_tools.util
def _RmTreeOnError(function, path, info):
  """Callback for handling shutil.rmtree errors."""
  # On Windows, file operation against a newly created or updated file
  # sometimes conflicts with that by security software and/or file indexer.
  time.sleep(0.5)
  if function == os.remove:
    os.chmod(path, stat.S_IREAD | stat.S_IWRITE)
    os.remove(path)
  elif function == os.listdir:
    os.chmod(path, stat.S_IREAD | stat.S_IWRITE | stat.S_IEXEC)
    shutil.rmtree(path, ignore_errors=True)
  elif function == os.rmdir and info[1].errno == errno.ENOTEMPTY:
    # Another race condition? Retry.
    shutil.rmtree(path, ignore_errors=True)


# TODO(team): Move this to build_tools.util
class PathDeleter(object):
  """A deleter to ensure that the given path object is certainly removed."""

  def __init__(self, path):
    """Stores the target path."""
    self._path = path

  def __enter__(self):
    """Does nothing."""
    pass

  def __exit__(self, *unused_exc_info):
    """Recursively removes the files and directories under given path."""
    if not os.path.exists(self._path):
      return
    shutil.rmtree(self._path, onerror=_RmTreeOnError)
    if os.path.exists(self._path):
      # Try again without error handler after 1 sec sleep.
      time.sleep(1)
      try:
        shutil.rmtree(self._path)
      except OSError as e:
        logging.error('Failed to remove %s. error: %s', self._path, e)


def _ExecuteTest(command_and_gtest_report_dir):
  """Executes tests with specified Test command.

  Args:
    command_and_gtest_report_dir: tuple of (command, gtest_report_dir) where
        command is a list of string to be executed.  gtest_report_dir is the
        directory path where gtest XML reports will be stored.

  Returns:
    A dictionary:
      command: An performed command-line string list.
      result: An boolean which represents test result.

  This should be a top-level function due to the restriction of the pickle
  module, which is used in multiprocessing module.
  (http://docs.python.org/library/pickle.html)
  """
  command, gtest_report_dir = command_and_gtest_report_dir
  binary = command[0]
  binary_filename = os.path.basename(binary)
  tmp_dir = tempfile.mkdtemp()
  with PathDeleter(tmp_dir):
    tmp_xml_path = os.path.join(tmp_dir, '%s.xml' % binary_filename)
    # Due to incompatibility between the prefixes of internal and external,
    # testing libraries.
    test_command = command + ['--test_tmpdir=%s' % tmp_dir,
                              '--gunit_output=xml:%s' % tmp_xml_path,
                              '--gtest_output=xml:%s' % tmp_xml_path]
    try:
      proc = subprocess.Popen(test_command, stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT)
      (output, _) = proc.communicate()
      result = proc.poll() == 0 and os.path.isfile(tmp_xml_path)
      if os.path.isfile(tmp_xml_path) and gtest_report_dir:
        shutil.copy(tmp_xml_path, gtest_report_dir)
    except OSError:
      logging.fatal('Fail to execute %s', ' '.join(command))
      sys.exit(1)

  if result:
    label = util.ColoredText('[ PASSED ]', logging.INFO)
    logging.info('%s %s', label, binary)
  else:
    label = util.ColoredText('[ FAILED ]', logging.ERROR)
    logging.error('Failed. Detail output:\n%s', output.decode('ascii'))
    logging.info('%s %s', label, binary)

  return {'command': command, 'result': result}


class TestLauncher(object):
  """This class provieds task parallelization.

  This class accepts command-line strings and executes them in parallel.
  """

  def __init__(self, gtest_report_dir=''):
    """Initializes the object.

    Args:
      gtest_report_dir: Directory path where gtest XML reports will be stored.
    """
    self._test_commands = []
    self._gtest_report_dir = gtest_report_dir

  def Execute(self, num_parallel):
    """Starts to run tests.

    This method watches all test command until all of them are finished.

    Args:
      num_parallel: Allows "num_parallel" tasks at once.
    Returns:
      An failed command list. Each command represents as one-line
      command-line string. If this list is empty, all tests are passed.
    """
    # TODO(nona): Show progress report for debugging.
    try:
      pool = multiprocessing.Pool(processes=num_parallel)
      params = [(command, self._gtest_report_dir)
                for command in self._test_commands]
      # Workaround against http://bugs.python.org/issue8296
      # See also
      # http://stackoverflow.com/questions/1408356/keyboard-interrupts-with-pythons-multiprocessing-pool
      async_results = pool.map_async(_ExecuteTest, params)
      while True:
        try:
          results = async_results.get(1000000)
          break
        except multiprocessing.TimeoutError:
          pass
      pool.close()
      return [' '.join(result['command'])
              for result in results if not result['result']]
    except:
      pool.terminate()
      logging.fatal('Exception occurred.')
      raise

  def AddTestCommand(self, command):
    self._test_commands.append(command)



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Capture stdout output from a program and save it to a file.

  % python redirect.py out.txt echo foo
  % cat out.txt
  foo

This is essentially identical to 'echo foo > out.txt' but the script
can be used in platforms without a Unix shell (i.e. windows).
"""

from __future__ import absolute_import
from __future__ import print_function

import subprocess
import sys


def main():
  """The main function."""
  # Delete the self program name (i.e. redirect.py).
  sys.argv.pop(0)
  # The first parameter is the output file name.
  output_file_name = sys.argv.pop(0)
  # Rest are the target program name and the parameters, but we special
  # case if the target program name ends with '.py'
  if sys.argv[0].endswith('.py'):
    sys.argv.insert(0, sys.executable)  # Inject the python interpreter path.
  try:
    process = subprocess.Popen(sys.argv, stdout=subprocess.PIPE)
  except:
    print('==========')
    print(' ERROR: %s' % ' '.join(sys.argv))
    print('==========')
    raise
  (stdout_content, _) = process.communicate()
  # Write the stdout content to the output file.
  with open(output_file_name, 'wb') as output_file:
    output_file.write(stdout_content)
  return process.wait()

if __name__ == '__main__':
  sys.exit(main())



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""A tool to generate test sentences for stress test."""

from __future__ import print_function

import codecs
import sys
import six


def escape_string(s):
  r"""escape the string with "\\xXX" format.

  We don't use encode('string_escape') because it doesn't escape ascii
  characters.

  Args:
    s: a string to be escaped

  Returns:
    an escaped string.
  """
  if six.PY3:
    return ''.join(r'\x%02x' % b for b in s.encode('utf-8'))

  result = ''
  for c in s:
    hexstr = hex(ord(c))
    # because hexstr contains '0x', remove the prefix and add our prefix
    result += '\\x' + hexstr[2:]
  return result


def OpenFile(filename):
  if six.PY2:
    return open(filename, 'r')
  else:
    return codecs.open(filename, 'r', encoding='utf-8')


def GenerateHeader(file):
  try:
    print('const char *kTestSentences[] = {')
    for line in OpenFile(file):
      if line.startswith('#'):
        continue
      line = line.rstrip('\r\n')
      if not line:
        continue
      print(' "%s",' % escape_string(line))
    print('};')
  except Exception as e:
    print('cannot open %s' % file)
    raise e


def main():
  GenerateHeader(sys.argv[1])

if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Fix @@@...@@@ format of variables in the installer script templates for Mac.

  % python tweak_macinstaller_script.py --output=out.txt --input=in.txt \
    --version_file=version.txt [--build_type=dev] \
"""

__author__ = "mukai"

import logging
import optparse

import mozc_version


def _ReplaceVariables(data, environment):
  """Replace all occurrence of the variable in data by the value.

  Args:
    data: the original data string
    environment: an iterable of (variable name, its value) pairs

  Returns:
    the data string which replaces the variables by the value.
  """
  result = data
  for (k, v) in environment:
    result = result.replace(k, v)
  return result


def ParseOptions():
  """Parse command line options.

  Returns:
    An options data.
  """
  parser = optparse.OptionParser()
  parser.add_option('--version_file', dest='version_file')
  parser.add_option('--output', dest='output')
  parser.add_option('--input', dest='input')
  parser.add_option('--build_type', dest='build_type')

  (options, unused_args) = parser.parse_args()
  return options


def main():
  """The main function."""
  options = ParseOptions()
  required_flags = ['version_file', 'output', 'input']
  for flag in required_flags:
    if getattr(options, flag) is None:
      logging.error('--%s is not specified.', flag)
      exit(-1)

  version = mozc_version.MozcVersion(options.version_file)

  if options.build_type == 'dev':
    omaha_tag = 'external-dev'
  else:
    omaha_tag = 'external-stable'

  # This definition is copied from tools/scons/script.py
  variables = [
      ('@@@MOZC_VERSION@@@', version.GetVersionString()),
      ('@@@MOZC_PRODUCT_ID@@@', 'com.google.JapaneseIME'),
      ('@@@MOZC_APP_PATH@@@', '/Library/Input Methods/GoogleJapaneseInput.app'),
      ('@@@MOZC_APPLICATIONS_DIR@@@',
       '/Applications/GoogleJapaneseInput.localized'),
      ('@@@MOZC_OMAHA_TAG@@@', omaha_tag),
      ('@@@MOZC_PACKAGE_NAME@@@', 'GoogleJapaneseInput.pkg'),
      ]

  open(options.output, 'w').write(
      _ReplaceVariables(open(options.input).read(), variables))

if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Replaces macros in a text file with given values.

For example,

  % python replace_macros.py --input=in.txt --output=out.txt \
      --define=enable_foo=1 --define=enable_bar=0 --define=baz=str

replaces @FOO@ with true, @BAR@ with false, and @BAZ@ with str.

There are following special prefixes.  Prefixes are not considered as
part of variable name.
  enable_ = The value is treated as a boolean.  No value defaults to true.
  disable_ = The value is treated as a boolean.  No value defaults to false.
  squoted_ = The value is treated as a string and single quotes in the value
      are escaped with backslash.  The value is surrounded by single quotes.
  dquoted_ = The value is treated as a string and double quotes in the value
      are escaped with backslash.  The value is surrounded by double quotes.
"""

__author__ = "yukishiino"


import optparse


def ParseOptions():
  """Parses command line options.

  Returns:
    Parsed options and arguments.
  """
  parser = optparse.OptionParser()
  parser.add_option('--input', dest='input')
  parser.add_option('--output', dest='output')
  parser.add_option('--define', dest='variables', action='append', default=[])

  return parser.parse_args()


def ParseVariableDefinitions(var_defs):
  """Parses variable definitions.

  Args:
    var_defs: A list of strings which define variables in the form of
        "[prefix_]var_name[=value]".

  Returns:
    A list in the form of [(prefix, var_name, value), ...].
  """
  prefixes = ('enable_', 'disable_', 'squoted_', 'dquoted_')

  def _TakeValue(var_value):
    var_value = var_value.split('=', 1)
    if len(var_value) == 1:
      return (var_value[0], None)
    else:
      return (var_value[0], var_value[1])

  def _SplitToPrefixVarNameAndValue(var_def):
    for prefix in prefixes:
      if var_def.startswith(prefix):
        return (prefix,) + _TakeValue(var_def[len(prefix):])
    else:
      return (None,) + _TakeValue(var_def)

  def _ParseVarDef(var_def):
    prefix, var, value = _SplitToPrefixVarNameAndValue(var_def)

    # Interpret a value from the value itself.
    if value is None:
      pass
    elif value.isdigit():
      value = int(value)
    elif value.lower() == 'true':
      value = True
    elif value.lower() == 'false':
      value = False

    # Re-interpret a value from the prefix.
    if prefix == 'enable_':
      value = True if value is None else bool(value)
    elif prefix == 'disable_':
      prefix = 'enable_'
      value = False if value is None else not bool(value)
    elif prefix == 'squoted_' and value is not None:
      value = '\'' + str(value).replace('\'', '\\\'') + '\''
    elif prefix == 'dquoted_' and value is not None:
      value = '"' + str(value).replace('"', '\\"') + '"'

    return (prefix, var, value)

  return [_ParseVarDef(var_def) for var_def in var_defs]


def TransformValuesToCStyle(variables):
  """Returns variables with values replaced with strings suitable for print.

  Args:
    variables: A list in the form of [(prefix, var_name, values), ...].

  Returns:
    The same list as |variables| but some of values are replaced with
    printable strings.
  """

  def _ToCStyle(value):
    if type(value) is bool:
      return 'true' if value else 'false'
    else:
      return value

  return [(prefix, var, _ToCStyle(value)) for prefix, var, value in variables]


def ReplaceVariables(text, variables):
  """Returns the |text| with variables replaced with their values.

  Args:
    text: A string to be replaced.
    variables: A list in the form of [(prefix, var_name, values), ...].

  Returns:
    A replaced string.
  """
  for unused_prefix, var_name, value in variables:
    if var_name:
      text = text.replace('@%s@' % var_name.upper(), str(value))

  return text


def main():
  """The main function."""
  (options, unused_args) = ParseOptions()
  if unused_args:
    raise RuntimeError('too many arguments')

  if options.input is None:
    raise RuntimeError('no --input option')
  if options.output is None:
    raise RuntimeError('no --output option')

  variables = ParseVariableDefinitions(options.variables)

  if options.input.endswith('.py') or options.output.endswith('.py'):
    pass
  else:
    variables = TransformValuesToCStyle(variables)

  open(options.output, 'w').write(
      ReplaceVariables(open(options.input, 'r').read(),
                       variables))


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Script to invoke protoc with considering project root directory.

  % python protoc_wrapper.py               \
      --protoc_command=protoc              \
      --protoc_dir=/usr/bin    (optional)  \
      --proto=../protos/my_data.proto      \
      --proto_path=../protos   (optional)  \
      --cpp_out=../out/debug/gen           \
      --project_root=../
"""

__author__ = "yukawa"

import optparse
import os
import subprocess
import sys

def ParseOption():
  """Parse command line options."""
  parser = optparse.OptionParser()
  parser.add_option('--protoc_command', dest='protoc_command',
                    help='executable name of protoc')
  parser.add_option('--protoc_dir', dest='protoc_dir',
                    help='directory where protoc is located')
  parser.add_option('--proto', dest='proto', help='path of the *.proto file')
  parser.add_option('--cpp_out', dest='cpp_out', default='',
                    help='path where cpp files should be generated')
  parser.add_option('--project_root', dest='project_root', default='.',
                    help='run protoc after moving this directory')
  parser.add_option('--proto_path', dest='proto_path', default='',
                    help='directory to be passed to --proto_path option')

  (opts, _) = parser.parse_args()

  return opts


def main():
  """The main function."""
  opts = ParseOption()

  # Convert to absolute paths before changing the current directory.
  project_root = os.path.abspath(opts.project_root)
  proto_path = os.path.abspath(opts.proto_path) if opts.proto_path else ''
  cpp_out = os.path.abspath(opts.cpp_out) if opts.cpp_out else ''

  protoc_path = opts.protoc_command
  if opts.protoc_dir:
    protoc_path = os.path.join(os.path.abspath(opts.protoc_dir), protoc_path)

  # The path of proto file should be transformed as a relative path from
  # the project root so that correct relative paths should be embedded into
  # generated files.
  proto_files = [os.path.relpath(os.path.abspath(p), project_root)
                 for p in opts.proto.split(' ')]

  # Move to the project root.
  os.chdir(project_root)

  commands = [protoc_path] + proto_files
  if cpp_out:
    commands += ['--cpp_out=' + cpp_out]
  if proto_path:
    rel_proto_path = os.path.relpath(proto_path, project_root)
    commands += ['--proto_path=' + rel_proto_path]
  sys.exit(subprocess.call(commands))


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

r"""Zip code dictionary generator.

 The tool for generating zip code dictionary.
 Input files are shift-jis csv.
 Output lines will be printed as utf-8.

 usage:
   PYTHONPATH="${PYTHONPATH}:<Mozc's src root dir>"  \
     python ./gen_zip_code_seed.py  \
     --zip_code=zip_code.csv --jigyosyo=jigyosyo.csv  \
     > zip_code_seed.tsv

 Zip code sample input line:
 01101,"060  ","0600007","ﾎｯｶｲﾄﾞｳ",
 "ｻｯﾎﾟﾛｼﾁｭｳｵｳｸ","ｷﾀ7ｼﾞｮｳﾆｼ","北海道",
 "札幌市中央区","北七条西",0

 Jigyosyo zip code sample input line:
 01101,"ｻﾂﾎﾟﾛｼﾁﾕｳｵｳｸﾔｸｼﾖ",
 "札幌市中央区役所","北海道","札幌市中央区",
 "南三条西","１１丁目","0608612","060  ","札幌",0,0,0
"""

from __future__ import print_function
__author__ = "toshiyuki"

import optparse
import re
import sys
import unicodedata
import six

from dictionary import zip_code_util


ZIP_CODE_LABEL = 'ZIP_CODE'
ZIP_CODE_COST = 7000


class ZipEntry(object):
  """Container class for zip code entry."""

  def __init__(self, zip_code, address):
    self.zip_code = zip_code
    self.address = address

  def FormatZip(self, zip_code):
    """Get formatted zip code."""
    # XXX-XXXX format
    return '-'.join([zip_code[0:3], zip_code[3:]])

  def Output(self):
    """Output entry."""
    zip_code = self.FormatZip(self.zip_code)
    address = unicodedata.normalize('NFKC', self.address)
    line = '\t'.join([zip_code, '0', '0', str(ZIP_CODE_COST),
                      address, ZIP_CODE_LABEL])
    print(six.ensure_str(line))


def ProcessZipCodeCSV(file_name):
  """Process zip code csv."""
  csv_lines = zip_code_util.ReadCSV(file_name)
  merged_csv_lines = zip_code_util.MergeCSV(csv_lines)
  for tokens in merged_csv_lines:
    for entry in ReadZipCodeEntries(tokens[2], tokens[6], tokens[7], tokens[8]):
      entry.Output()


def ProcessJigyosyoCSV(file_name):
  """Process jigyosyo csv."""
  for tokens in zip_code_util.ReadCSV(file_name):
    entry = ReadJigyosyoEntry(tokens[7], tokens[3], tokens[4],
                              tokens[5], tokens[2])
    entry.Output()


def ReadZipCodeEntries(zip_code, level1, level2, level3):
  """Read zip code entries."""
  return [ZipEntry(zip_code, u''.join([level1, level2, town]))
          for town in ParseTownName(level3)]


def ReadJigyosyoEntry(zip_code, level1, level2, level3, name):
  """Read jigyosyo entry."""
  return ZipEntry(zip_code,
                  u''.join([level1, level2, level3, u' ', name]))


def ParseTownName(level3):
  """Parse town name."""
  if level3.find(u'以下に掲載がない場合') != -1:
    return ['']

  assert CanParseAddress(level3), ('failed to be merged %s'
                                   % level3.encode('utf-8'))

  # We ignore additional information here.
  level3 = re.sub(u'（.*）', u'', level3, re.U)

  # For 地割, we have these cases.
  #  XX1地割
  #  XX1地割、XX2地割、
  #  XX1地割〜XX2地割、
  #  XX第1地割
  #  XX第1地割、XX第2地割、
  #  XX第1地割〜XX第2地割、
  # We simply use XX for them.
  chiwari_match = re.match(u'(\\D*?)第?\\d+地割.*', level3, re.U)
  if chiwari_match:
    town = chiwari_match.group(1)
    return [town]

  # For "、"
  #  XX町YY、ZZ
  #   -> XX町YY and (XX町)ZZ
  #  YY、ZZ
  #   -> YY and ZZ
  chou_match = re.match(u'(.*町)?(.*)', level3, re.U)
  if chou_match:
    chou = u''
    if chou_match.group(1):
      chou = chou_match.group(1)
    rests = chou_match.group(2)
    return [chou + rest for rest in rests.split(u'、')]

  return [level3]


def CanParseAddress(address):
  """Return true for valid address."""
  return (address.find(u'（') == -1 or
          address.find(u'）') != -1)


def ParseOptions():
  """Parse command line options."""
  parser = optparse.OptionParser(usage='Usage: %prog [options]')
  parser.add_option('--zip_code', dest='zip_code',
                    action='store', default='',
                    help='specify zip code csv file path.')
  parser.add_option('--jigyosyo', dest='jigyosyo',
                    action='store', default='',
                    help='specify zip code csv file path.')
  (options, unused_args) = parser.parse_args()
  return options


def main():
  options = ParseOptions()

  if options.zip_code:
    ProcessZipCodeCSV(options.zip_code)

  if options.jigyosyo:
    ProcessJigyosyoCSV(options.jigyosyo)

  return 0


if __name__ == '__main__':
  sys.exit(main())



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

r"""Copy Qt frameworks to the target application's frameworks directory.

Typical usage:

  % python copy_qt_frameworks.py --qtdir=/path/to/qtdir/ \
      --target=/path/to/target.app/Contents/Frameworks/
"""

__author__ = "horo"

import optparse
import os
from copy_file import CopyFiles
from util import PrintErrorAndExit
from util import RunOrDie


def ParseOption():
  """Parse command line options."""
  parser = optparse.OptionParser()
  parser.add_option('--qtdir', dest='qtdir')
  parser.add_option('--target', dest='target')

  (opts, _) = parser.parse_args()

  return opts


def GetFrameworkPath(name):
  """Return path to the library in the framework."""
  return '%s.framework/Versions/5/%s' % (name, name)


def Symlink(src, dst):
  if os.path.exists(dst):
    return
  os.symlink(src, dst)


def CopyQt(qtdir, qtlib, target):
  """Copy a Qt framework from qtdir to target."""
  srcdir = '%s/lib/%s.framework/Versions/5/' % (qtdir, qtlib)
  dstdir = '%s/%s.framework/' % (target, qtlib)

  # This function creates the following file, directory and symbolic links.
  #
  # QtCore.framework/Versions/5/QtCore
  # QtCore.framework/Versions/5/Resources/
  # QtCore.framwwork/QtCore@ -> Versions/5/QtCore
  # QtCore.framwwork/Resources@ -> Versions/5/Resources
  # QtCore.framework/Versions/Current -> 5

  # cp {qtdir}/lib/QtCore.framework/Versions/5/QtCore
  #    {target}/QtCore.framework/Versions/5/QtCore
  CopyFiles([srcdir + qtlib], dstdir + 'Versions/5/' + qtlib)

  # Copies Resources of QtGui
  # cp -r {qtdir}/lib/QtCore.framework/Resources
  #       {target}/QtCore.framework/Versions/5/Resources
  CopyFiles([srcdir + 'Resources'],
            dstdir + 'Versions/5/Resources',
            recursive=True)

  # ln -s 5 {target}/QtCore.framework/Versions/Current
  Symlink('5', dstdir + 'Versions/Current')

  # ln -s Versions/Current/QtCore {target}/QtCore.framework/QtCore
  Symlink('Versions/Current/' + qtlib, dstdir + qtlib)

  # ln -s Versions/Current/Resources {target}/QtCore.framework/Resources
  Symlink('Versions/Current/Resources', dstdir + 'Resources')


def ChangeReferences(path, target, ref_to, references=None):
  """Change the references of frameworks, by using install_name_tool."""
  # Change id
  cmd = ['install_name_tool',
         '-id', '%s/%s' % (ref_to, path),
         '%s/%s' % (target, path)]
  RunOrDie(cmd)

  if not references:
    return

  # Change references
  for ref in references:
    ref_framework_path = GetFrameworkPath(ref)
    change_cmd = ['install_name_tool', '-change',
                  '@rpath/%s' % ref_framework_path,
                  '%s/%s' % (ref_to, ref_framework_path),
                  '%s/%s' % (target, path)]
    RunOrDie(change_cmd)


def main():
  opt = ParseOption()

  if not opt.qtdir:
    PrintErrorAndExit('--qtdir option is mandatory.')

  if not opt.target:
    PrintErrorAndExit('--target option is mandatory.')

  qtdir = os.path.abspath(opt.qtdir)
  target = os.path.abspath(opt.target)

  ref_to = '@executable_path/../../../ConfigDialog.app/Contents/Frameworks'

  CopyQt(qtdir, 'QtCore', target)
  CopyQt(qtdir, 'QtGui', target)
  CopyQt(qtdir, 'QtWidgets', target)
  CopyQt(qtdir, 'QtPrintSupport', target)

  qtcore_fpath = GetFrameworkPath('QtCore')
  qtgui_fpath = GetFrameworkPath('QtGui')
  qtwidgets_fpath = GetFrameworkPath('QtWidgets')
  qtprint_fpath = GetFrameworkPath('QtPrintSupport')

  ChangeReferences(qtcore_fpath, target, ref_to)
  ChangeReferences(qtgui_fpath, target, ref_to, references=['QtCore'])
  ChangeReferences(
      qtwidgets_fpath, target, ref_to, references=['QtCore', 'QtGui'])
  ChangeReferences(
      qtprint_fpath,
      target,
      ref_to,
      references=['QtCore', 'QtGui', 'QtWidgets'])

  libqcocoa = 'QtCore.framework/Resources/plugins/platforms/libqcocoa.dylib'
  CopyFiles(['%s/plugins/platforms/libqcocoa.dylib' % qtdir],
            '%s/%s' % (target, libqcocoa))
  ChangeReferences(libqcocoa, target, ref_to,
                   references=['QtCore', 'QtGui',
                               'QtWidgets', 'QtPrintSupport'])


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

r"""Change the reference to frameworks.

Typical usage:

  % change_reference_mac.py --qtdir=/path/to/qtdir/ \
      --target=/path/to/target.app/Contents/MacOS/target
"""

__author__ = "horo"

import optparse
import os

from util import PrintErrorAndExit
from util import RunOrDie


def ParseOption():
  """Parse command line options."""
  parser = optparse.OptionParser()
  parser.add_option('--qtdir', dest='qtdir')
  parser.add_option('--target', dest='target')

  (opts, _) = parser.parse_args()

  return opts


def GetFrameworkPath(name, version):
  return '%s.framework/Versions/%s/%s' % (name, version, name)


def GetReferenceTo(framework):
  return ('@executable_path/../../../ConfigDialog.app/Contents/Frameworks/%s' %
          framework)


def InstallNameTool(target, reference_from, reference_to):
  cmd = ['install_name_tool', '-change', reference_from, reference_to, target]
  RunOrDie(cmd)


def main():
  opt = ParseOption()

  if not opt.qtdir:
    PrintErrorAndExit('--qtdir option is mandatory.')

  if not opt.target:
    PrintErrorAndExit('--target option is mandatory.')

  unused_qtdir = os.path.abspath(opt.qtdir)  # TODO(komatsu): remove this.
  target = os.path.abspath(opt.target)

  # Changes the reference to QtCore framework from the target application
  # From: @rpath/QtCore.framework/Versions/5/QtCore
  #   To: @executable_path/../../../MozcTool.app/Contents/Frameworks/...
  qtcore_framework = GetFrameworkPath('QtCore', '5')
  InstallNameTool(target,
                  '@rpath/%s' % qtcore_framework,
                  GetReferenceTo(qtcore_framework))

  # Changes the reference to QtGui framework from the target application
  qtgui_framework = GetFrameworkPath('QtGui', '5')
  InstallNameTool(target,
                  '@rpath/%s' % qtgui_framework,
                  GetReferenceTo(qtgui_framework))

  # Changes the reference to QtWidgets framework from the target application
  qtwidgets_framework = GetFrameworkPath('QtWidgets', '5')
  InstallNameTool(target,
                  '@rpath/%s' % qtwidgets_framework,
                  GetReferenceTo(qtwidgets_framework))

  # Change the reference to $(branding)Tool_lib from the target application
  # From: @executable_path/../Frameworks/MozcTool_lib.framework/...
  #   To: @executable_path/../../../ConfigDialog.app/Contents/Frameworks/...
  toollib_framework = GetFrameworkPath('GuiTool_lib', 'A')
  InstallNameTool(target,
                  '@executable_path/../Frameworks/%s' % toollib_framework,
                  GetReferenceTo(toollib_framework))


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Util module for Japanese postal(ZIP) code."""

import codecs


def ReadCSV(file_name):
  """Read CSV file."""
  # Do not use csv reader module because it does not support unicode
  return [
      GetCells(line)
      for line in codecs.open(file_name, 'r', 'cp932', errors='replace')
  ]


def GetCells(line):
  """Get cells."""
  # [A, B, C, ..] from "A","B ",C,..
  return [column.strip('"').strip() for column in line.strip().split(',')]


def MergeCSV(csv_lines):
  """Merge CSV."""
  # When the flag says a zip code have no multiple entry while we can see
  # multiple line for that zip code, we have to merge them.
  zip_count = {}
  ret = []
  for entry in csv_lines:
    zip_code = entry[2]
    zip_count[zip_code] = zip_count.get(zip_code, 0) + 1
    if not ShouldMerge(zip_count, entry):
      ret.append(entry)
    else:
      last_entry = ret[-1]
      last_entry[5] += entry[5]  # reading of '町域'
      last_entry[8] += entry[8]  # '町域'
  return ret


def ShouldMerge(zip_count, entry):
  """Return true if this entry should be merged to the previous entry."""
  zip_code = entry[2]
  flag_multi = (entry[12] == '1')
  should_merge = (zip_count[zip_code] > 1 and not flag_multi)
  should_merge_special = ShouldMergeSpecial(entry)
  return should_merge or should_merge_special


class SpecialMergeZip(object):
  """Container class for special zip code entry to be merged."""

  def __init__(self, zip_code, pref, city, towns):
    self.zip_code = zip_code
    self.pref = pref
    self.city = city
    self.towns = towns


_SPECIAL_CASES = [
    SpecialMergeZip(u'5900111', u'大阪府', u'堺市中区', [u'三原台']),
    SpecialMergeZip(u'8710046', u'大分県', u'中津市',
                    [u'金谷', u'西堀端', u'東堀端', u'古金谷']),
    SpecialMergeZip(u'9218046', u'石川県', u'金沢市',
                    [u'大桑町', u'三小牛町']),
    ]


def ShouldMergeSpecial(entry):
  """Return true for special cases to be merged."""
  zip_code = entry[2]
  level1 = entry[6]
  level2 = entry[7]
  level3 = entry[8]
  for special_case in _SPECIAL_CASES:
    if (zip_code == special_case.zip_code and
        level1 == special_case.pref and
        level2 == special_case.city and
        ContinuedLine(level3, special_case.towns)):
      return True
  return False


def ContinuedLine(level3, towns):
  """Return true if this seems continued line."""
  for town in towns:
    if level3.startswith(town):
      return False
  return True



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""A library to operate version definition file.

This script has two functionarity which relate to version definition file.

1. Generate version definition file from template and given parameters.
  To generate version definition file, use GenerateVersionFileFromTemplate
  method.

2. Parse (generated) version definition file.
  To parse, use MozcVersion class.

Typically version definition file is ${PROJECT_ROOT}/mozc_version.txt
(Not in the repository because it is generated by this script)
Typically version template file is
${PROJECT_ROOT}/data/version/mozc_version_template.bzl,
which is in the repository.
The syntax of template is written in the template file.
"""
# TODO(matsuzaki): MozcVersion class should have factory method which takes
#   file path and we should remove all the module methods instead to
#   simplify the design. Currently I'd keep this design to reduce
#   client side's change.

from __future__ import absolute_import
import datetime
import logging
import optparse
import os
import re
import sys


TARGET_PLATFORM_TO_DIGIT = {
    'Windows': '0',
    'Mac': '1',
    'Linux': '2',
    'Android': '3',
    'NaCl': '4',
    'iOS': '6',
    'iOS_sim': '6',
    'Wasm': '7',
    }

VERSION_PROPERTIES = [
    'MAJOR',
    'MINOR',
    'BUILD',
    'REVISION',
    'TARGET_PLATFORM',
    'QT_VERSION',
    'ENGINE_VERSION',
    'DATA_VERSION',
    ]

MOZC_EPOCH = datetime.date(2009, 5, 24)


def _GetRevisionForPlatform(revision, target_platform):
  """Returns the revision for the current platform."""
  if revision is None:
    logging.critical('REVISION property is not found in the template file')
    sys.exit(1)
  last_digit = TARGET_PLATFORM_TO_DIGIT.get(target_platform, None)
  if last_digit is None:
    logging.critical('target_platform %s is invalid. Accetable ones are %s',
                     target_platform, list(TARGET_PLATFORM_TO_DIGIT.keys()))
    sys.exit(1)

  if not revision:
    return revision

  if last_digit:
    return revision[0:-1] + last_digit

  # If not supported, just use the specified version.
  return revision


def _ParseVersionTemplateFile(template_path, target_platform,
                              qt_version, build_override):
  """Parses a version definition file.

  Args:
    template_path: A filename which has the version definition.
    target_platform: The target platform on which the programs run.
    qt_version: '4' for Qt4, '5' for Qt5, and '' or None for no-Qt.
    build_override: An optional string to override BUILD number.
  Returns:
    A dictionary generated from the template file.
  """
  template_dict = {}
  with open(template_path) as template_file:
    for line in template_file:
      matchobj = re.match(r'(\w+) *= *(\w*)', line.strip())
      if matchobj:
        var = matchobj.group(1)
        val = matchobj.group(2)
        if var in template_dict:
          logging.warning(('Dupulicate key: "%s". Later definition "%s"'
                           'overrides earlier one "%s".'),
                          var, val, template_dict[var])
        template_dict[var] = val

  # Some properties need to be tweaked.
  template_dict['REVISION'] = _GetRevisionForPlatform(
      template_dict.get('REVISION', None), target_platform)

  template_dict['TARGET_PLATFORM'] = target_platform
  template_dict['QT_VERSION'] = qt_version
  if build_override is not None:
    template_dict['BUILD'] = build_override
  return template_dict


def _GetVersionInFormat(properties, version_format):
  """Returns the version string based on the specified format.

  format can contains @MAJOR@, @MINOR@, @BUILD@ and @REVISION@ which are
  replaced by self._major, self._minor, self._build, and self._revision
  respectively.

  Args:
    properties: a property dicitonary. Typically gotten from
      _ParseVersionTemplateFile method.
    version_format: a string which contains version patterns.

  Returns:
    Return the version string in the format of format.
  """

  result = version_format
  for keyword in VERSION_PROPERTIES:
    result = result.replace('@%s@' % keyword, properties.get(keyword, ''))
  return result


def _GetChangelistNumber(build_override, build_changelist_file):
  """Returns the changelist number from the value or the file path."""
  if build_override:
    return build_override

  if not build_changelist_file:
    return None

  with open(build_changelist_file, 'r') as cl_file:
    for line in cl_file:
      if line.startswith('BUILD_CHANGELIST'):
        return line.rstrip().split(' ')[1]

  return None


def GenerateVersionFileFromTemplate(template_path,
                                    output_path,
                                    version_format,
                                    target_platform,
                                    qt_version='',
                                    build_override=None):
  """Generates version file from template file and given parameters.

  Args:
    template_path: A path to template file.
    output_path: A path to generated version file.
      If already exists and the content will not be updated, nothing is done
      (the timestamp is not updated).
    version_format: A string which contans version patterns.
    target_platform: The target platform on which the programs run.
    qt_version: '4' for Qt4, '5' for Qt5, and '' or None for no-Qt.
    build_override: An optional string to override BUILD number
      in the template.
  """

  properties = _ParseVersionTemplateFile(template_path, target_platform,
                                         qt_version, build_override)
  version_definition = _GetVersionInFormat(properties, version_format)
  old_content = ''
  if os.path.exists(output_path):
    # If the target file already exists, need to check the necessity of update
    # to reduce file-creation frequency.
    # Currently generated version file is not seen from Make (and Make like
    # tools) so recreation will not cause serious issue but just in case.
    with open(output_path) as output_file:
      old_content = output_file.read()

  if version_definition != old_content:
    with open(output_path, 'w') as output_file:
      output_file.write(version_definition)


def GenerateVersionFile(version_template_path, version_path, target_platform,
                        qt_version, build_override=None):
  """Reads the version template file and stores it into version_path.

  This doesn't update the "version_path" if nothing will be changed to
  reduce unnecessary build caused by file timestamp.

  Args:
    version_template_path: a file name which contains the template of version.
    version_path: a file name to be stored the official version.
    target_platform: target platform name. c.f. --target_platform option
    qt_version: '4' for Qt4, '5' for Qt5, and '' or None for no-Qt.
    build_override: an optional string to override BUILD number in the template.
  """
  version_format = '\n'.join([
      'MAJOR=@MAJOR@',
      'MINOR=@MINOR@',
      'BUILD=@BUILD@',
      'REVISION=@REVISION@',
      'TARGET_PLATFORM=@TARGET_PLATFORM@',
      'QT_VERSION=@QT_VERSION@',
      'ENGINE_VERSION=@ENGINE_VERSION@',
      'DATA_VERSION=@DATA_VERSION@',
  ]) + '\n'
  GenerateVersionFileFromTemplate(
      version_template_path,
      version_path,
      version_format,
      target_platform=target_platform,
      qt_version=qt_version,
      build_override=build_override)


class MozcVersion(object):
  """A class to parse and maintain the version definition data.

  Note that this class is not intended to parse "template" file but to
  "generated" file.
  Typical usage is;
    GenerateVersionFileFromTemplate(template_path, version_path, format)
    version = MozcVersion(version_path)
  """

  def __init__(self, path):
    """Parses a version definition file.

    Args:
      path: A filename which has the version definition.
            If the file is not existent, empty properties are prepared instead.
    """

    self._properties = {}
    if not os.path.isfile(path):
      return
    for line in open(path):
      matchobj = re.match(r'(\w+)=(.*)', line.strip())
      if matchobj:
        var = matchobj.group(1)
        val = matchobj.group(2)
        if var not in self._properties:
          self._properties[var] = val

    # Check mandatory properties.
    for key in VERSION_PROPERTIES:
      if key not in self._properties:
        # Don't raise error nor exit.
        # Error handling is the client's responsibility.
        logging.warning('Mandatory key "%s" does not exist in %s', key, path)

  def IsDevChannel(self):
    """Returns true if the parsed version is dev-channel."""
    revision = self._properties['REVISION']
    return revision is not None and len(revision) >= 3 and revision[-3] == '1'

  def GetTargetPlatform(self):
    """Returns the target platform.

    Returns:
      A string for target platform.
      If the version file is not existent, None is returned.
    """
    return self._properties.get('TARGET_PLATFORM', None)

  def GetQtVersion(self):
    """Returns the target Qt version.

    Returns:
      A string that indicates the Qt version.
      '4' for Qt4, '5' for Qt5, and '' for no-Qt.
    """
    return self._properties.get('QT_VERSION', None)

  def GetVersionString(self):
    """Returns the normal version info string.

    Returns:
      a string in format of "MAJOR.MINOR.BUILD.REVISION"
    """
    return self.GetVersionInFormat('@MAJOR@.@MINOR@.@BUILD@.@REVISION@')

  def GetVersionInFormat(self, version_format):
    """Returns the version string based on the specified format."""
    return _GetVersionInFormat(self._properties, version_format)


def main():
  """Generates version file based on the default format.

  Generated file is mozc_version.txt compatible.
  """
  parser = optparse.OptionParser(usage='Usage: %prog ')
  parser.add_option('--template_path', dest='template_path',
                    help='Path to a template version file.')
  parser.add_option('--output', dest='output',
                    help='Path to the output version file.')
  parser.add_option('--target_platform', dest='target_platform',
                    help='Target platform of the version info.')
  parser.add_option('--qtver', dest='qtver', choices=('4', '5', ''),
                    default='', help='Specifies Qt version (desktop only)')
  parser.add_option('--build_override', dest='build_override',
                    help='Overrides BUILD number in the template.')
  parser.add_option('--build_changelist_file', dest='build_changelist_file',
                    help='Filepath containing the BUILD number.')
  (options, args) = parser.parse_args()
  assert not args, 'Unexpected arguments.'
  assert options.template_path, 'No --template_path was specified.'
  assert options.output, 'No --output was specified.'
  assert options.target_platform, 'No --target_platform was specified.'

  cl_number = _GetChangelistNumber(options.build_override,
                                   options.build_changelist_file)
  GenerateVersionFile(
      version_template_path=options.template_path,
      version_path=options.output,
      target_platform=options.target_platform,
      qt_version=options.qtver,
      build_override=cl_number)

if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Generate a C++ include file for testing data from TSV files."""

from __future__ import print_function

import codecs
import logging
import sys


def escape(string):
  return ''.join('\\x%02x' % char for char in string.encode('utf-8'))


def convert_tsv(filename):
  """Loads a TSV file and returns an entry of TestCase."""
  tsv = codecs.open(filename, 'rb', 'utf-8')
  for line in tsv:
    line = line.rstrip()
    if not line or line.startswith('#'):
      continue

    fields = line.split('\t')
    if len(fields) == 3:
      label = fields[0]
      expected = fields[1]
      query = fields[2]
    elif len(fields) < 6:
      logging.warning('invalid row format: %s', line)
      continue
    else:
      label = fields[0]
      expected = fields[4]
      query = fields[5]

    print('  // {"%s", "%s", "%s"},' % (label, expected, query))
    print('  {"%s", "%s", "%s"},' %
          (escape(label), escape(expected), escape(query)))
  tsv.close()


def main():
  sys.stdin = codecs.getreader('utf-8')(sys.stdin.buffer)
  sys.stdout = codecs.getwriter('utf-8')(sys.stdout.buffer)
  sys.stderr = codecs.getwriter('utf-8')(sys.stderr.buffer)
  logging.basicConfig(level=logging.INFO)

  print('// Automatically generated by mozc')
  print('#ifndef MOZC_SESSION_QUALITY_MAIN_DATA_H_')
  print('#define MOZC_SESSION_QUALITY_MAIN_DATA_H_')
  print('')
  print('namespace mozc {')
  print('struct TestCase {')
  print('  const char* source;')
  print('  const char* expected_result;')
  print('  const char* hiragana_sentence;')
  print('};')
  print('')
  print('static TestCase test_cases[] = {')

  for filename in sys.argv[1:]:
    convert_tsv(filename)

  print('  {nullptr, nullptr, nullptr}')
  print('};')
  print('}  // namespace mozc')
  print('#endif  // MOZC_SESSION_QUALITY_MAIN_DATA_H_')


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""A tool to embedded tsv file into test binary for quality regression test."""

from __future__ import absolute_import
from __future__ import print_function

import codecs
import sys
import xml.dom.minidom


def EscapeString(s):
  r"""escape the string with "\xXX" format.

  We don't use encode('string_escape') because it doesn't escape ascii
  characters.

  Args:
    s: a string to be escaped

  Returns:
    an escaped string.
  """
  result = ''
  for c in s:
    hexstr = hex(ord(c))
    # because hexstr contains '0x', remove the prefix and add our prefix
    result += '\\x' + hexstr[2:]
  return result

_DISABLED = 'false'
_ENABLED = 'true'


def ParseTSV(file):
  for line in codecs.open(file, 'r', encoding='utf-8'):
    if line.startswith('#'):
      continue
    line = line.rstrip('\r\n')
    if not line:
      continue
    yield (_ENABLED, line)


def GetText(node):
  if len(node) >= 1 and node[0].firstChild:
    return node[0].firstChild.nodeValue.strip().replace('\t', ' ')
  else:
    return ''


def ParseXML(file):
  dom = xml.dom.minidom.parse(file)
  for issue in dom.getElementsByTagName('issue'):
    status = GetText(issue.getElementsByTagName('status'))
    enabled = (_DISABLED if status != 'Fixed' and status != 'Verified'
               else _ENABLED)
    id = issue.attributes['id'].value
    target = GetText(issue.getElementsByTagName('target'))
    for detail in issue.getElementsByTagName(u'detail'):
      fields = []
      fields.append('mozcsu_%s' % id)
      for key in ('reading', 'output', 'actionStatus', 'rank', 'accuracy'):
        fields.append(GetText(detail.getElementsByTagName(key)))
      if target:
        fields.append(target)
      tsv_line = ('\t'.join(fields)).encode('utf-8')
      yield (enabled, tsv_line)


def ParseFile(file):
  if file.endswith('.xml'):
    return ParseXML(file)
  else:
    return ParseTSV(file)


def GenerateHeader(files):
  print('namespace mozc{')
  print('struct TestCase {')
  print('  const bool enabled;')
  print('  const char *tsv;')
  print('} kTestData[] = {')

  for file in files:
    try:
      for enabled, line in ParseFile(file):
        print(' {%s, "%s"},' % (enabled, EscapeString(line)))
    except:
      print('cannot open %s' % file)
      sys.exit(1)

  print('  {false, nullptr},')
  print('};')
  print('}  // namespace mozc')


def main():
  GenerateHeader(sys.argv[1:])

if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Generates usage stats name list."""

from __future__ import print_function

__author__ = "toshiyuki"

import codecs
import sys


def GetStatsNameList(filename):
  stats = []
  for line in codecs.open(filename, 'r', encoding='utf-8'):
    stat = line.strip()
    if not stat or stat[0] == '#':
      continue
    stats.append(stat)
  return stats


def main():
  stats_list = GetStatsNameList(sys.argv[1])
  print('// This header file is generated by gen_stats_list.py')
  for stats in stats_list:
    print('const char k%s[] = "%s";' % (stats, stats))
  print('const char *kStatsList[] = {')
  for stats in stats_list:
    print('  k%s,' % (stats))
  print('};')


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Generates touch event's data stream file from csv file.

Typical usage is:
python android/gen_touch_event_stat.py --output_dir android/assets \
--stats_data data/typing/touch_event_stats.csv \
--collected_keyboards android/collected_keyboards.csv
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from collections import defaultdict
import csv
import optparse
import os
import struct

import six
from six.moves import urllib

__author__ = "matsuzakit"


def ReadCollectedKeyboards(stream):
  """Parses collected keyboard list (CSV).

  Args:
    stream: An input stream.
  Returns:
    A frozeset of tuples (base_name, major, minor, revision).
  """
  result = set()
  reader = csv.DictReader(stream)
  for row in reader:
    result.add((row['base_name'], row['major'], row['minor'], row['revision']))
  return frozenset(result)


def ReadData(stream, collected_keyboards):
  """Reads CSV file of the stats and returns the result as dictionary.

  Args:
    stream: An input stream of the file.
    collected_keyboards: A collection of the keyboard spec tuple
                         (base_name, major, minor, revision)
                         of which data is used.
  Returns:
    A dictionary: base_name_orientation -> source_id -> stats_type ->
                  (cumulative_average, count)
  """
  reader = csv.DictReader(stream)
  stats = defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: (0, 0))))
  for row in reader:
    key = (row['base_name'], row['major'], row['minor'], row['revision'])
    if not key in collected_keyboards:
      continue
    base_name_orientation = (row['base_name'], row['orientation'])
    source_id = int(row['source_id'])
    stats_type = row['stats_type']
    (cumulative_average, count) = (
        stats[base_name_orientation][source_id][stats_type])
    new_value = (cumulative_average + int(row['sum']),
                 count + int(row['count']))
    stats[base_name_orientation][source_id][stats_type] = new_value
  return stats


def WriteKeyboardData(keyboard_value, stream):
  """Writes serialized binary of the stats for a keyboard.

  Args:
    keyboard_value: A dictionary from souce_id to source_value.
                    source_value is a dictionary from stats_type to
                    (cumulative_average, count).
    stream: An output stream.
  """

  def GetAverage(source_value, stats_type):
    (cumulative_average, count) = source_value[stats_type]
    if count == 0:
      return 0
    # Cumulative average values are multiplied by 10^7
    # when usage stats are sent to the log server.
    # Here we get original value by dividing the avarage value.
    return cumulative_average / 1e7 / count

  # c.f. usage_stats/usage_stats_uploader.cc
  keys = ('sxa', 'sya', 'sxv', 'syv', 'dxa', 'dya', 'dxv', 'dyv')
  stream.write(struct.pack('>i', len(keyboard_value)))
  for source_id, source_value in six.iteritems(keyboard_value):
    stream.write(struct.pack('>i', source_id))
    # Note that we are calculating
    # "Average of average" and "Average of variance".
    # Such values are theoritically meaningless but we collect
    # only restricted value to avoid from privacy issue
    # so we have to use them instead.
    stream.write(struct.pack('>ffffffff',
                             *[GetAverage(source_value, key) for key in keys]))


def WriteData(stats, output_dir):
  for base_name_orientation in six.iterkeys(stats):
    with open(
        os.path.join(
            output_dir, '%s_%s.touch_stats' % (urllib.parse.unquote(
                base_name_orientation[0]), base_name_orientation[1])),
        'wb') as stream:
      WriteKeyboardData(stats[base_name_orientation], stream)


def ParseOptions():
  parser = optparse.OptionParser()
  parser.add_option('--collected_keyboards', dest='collected_keyboards',
                    help='Keyboards to be collected')
  parser.add_option('--stats_data', dest='stats_data',
                    help='Path to touch_event_stats.csv')
  parser.add_option('--output_dir', dest='output_dir', help='Output file name')
  return parser.parse_args()[0]


def main():
  options = ParseOptions()
  with open(options.collected_keyboards) as stream:
    collected_keyboards = ReadCollectedKeyboards(stream)
  with open(options.stats_data) as stream:
    stats = ReadData(stream, collected_keyboards)
  output_dir = options.output_dir
  if not os.path.exists(output_dir):
    os.makedirs(output_dir)
  WriteData(stats, output_dir)

if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""A script to ensure 'import gyp' loads a gyp module from expected location.

Example:
./ensure_gyp_module_path.py
    --expected=/home/foobar/work/mozc/src/third_party/gyp/pylib/gyp
"""

from __future__ import absolute_import
from __future__ import print_function

import optparse
import os
import sys


def ParseOption():
  """Parse command line options."""
  parser = optparse.OptionParser()
  parser.add_option('--expected', dest='expected')

  (options, _) = parser.parse_args()
  if not options.expected:
    print(parser.print_help())
    sys.exit(1)

  return options


def main():
  """Script to ensure gyp module location."""
  opt = ParseOption()
  expected_path = os.path.abspath(opt.expected)
  if not os.path.exists(expected_path):
    print('%s does not exist.' % expected_path)
    sys.exit(1)

  try:
    import gyp  # NOLINT
  except ImportError as e:
    print('import gyp failed: %s' % e)
    sys.exit(1)

  actual_path = os.path.abspath(gyp.__path__[0])
  if expected_path != actual_path:
    print('Unexpected gyp module is loaded on this environment.')
    print('  expected: %s' % expected_path)
    print('  actual  : %s' % actual_path)
    sys.exit(1)

if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Pre-generate XML data of ibus-mozc engines.

We don't support --xml command line option in the ibus-engines-mozc command
since it could make start-up time of ibus-daemon slow when XML cache of the
daemon in ~/.cache/ibus/bus/ is not ready or is expired.
"""

from __future__ import print_function
__author__ = "yusukes"

import optparse
import os
import sys

# Information to generate <component> part of mozc.xml. %s will be replaced with
# a product name, 'Mozc' or 'Google Japanese Input'.
IBUS_COMPONENT_PROPS = {
    'name': 'com.google.IBus.Mozc',
    'description': '%(product_name)s Component',
    'exec': '%(ibus_mozc_path)s --ibus',
    # TODO(mazda): Generate the version number.
    'version': '0.0.0.0',
    'author': 'Google Inc.',
    'license': 'New BSD',
    'homepage': 'https://github.com/google/mozc',
    'textdomain': 'ibus-mozc',
}

# A dictionary from --branding to a product name which is embedded into the
# properties above.
PRODUCT_NAMES = {
    'Mozc': 'Mozc',
    'GoogleJapaneseInput': 'Google Japanese Input',
}

CPP_HEADER = """// Copyright 2010 Google Inc. All Rights Reserved.

#ifndef %s
#define %s

#include <cstddef>

namespace {"""

CPP_FOOTER = """}  // namespace
#endif  // %s"""


def GetXmlElement(param_dict, element_name, value):
  return '  <%s>%s</%s>' % (element_name, (value % param_dict), element_name)


def GetTextProtoElement(param_dict, element_name, value):
  return '  %s : "%s"' % (element_name, (value % param_dict))


def GetEnginesXml(param_dict, engine_common, engines, setup_arg):
  """Outputs a XML data for ibus-daemon.

  Args:
    param_dict: A dictionary to embed options into output string.
        For example, {'product_name': 'Mozc'}.
    engine_common: A dictionary from a property name to a property value that
        are commonly used in all engines. For example, {'language': 'ja'}.
    engines: A dictionary from a property name to a list of property values of
        engines. For example, {'name': ['mozc-jp', 'mozc', 'mozc-dv']}.
    setup_arg: A command line to execute Mozc's configuration tool.
  Returns:
    output string in XML.
  """
  output = ['<engines>']
  for i in range(len(engines['name'])):
    output.append('<engine>')
    for key in engine_common:
      output.append(GetXmlElement(param_dict, key, engine_common[key]))
    if setup_arg:
      output.append(GetXmlElement(param_dict, 'setup', ' '.join(setup_arg)))
    for key in engines:
      output.append(GetXmlElement(param_dict, key, engines[key][i]))
    output.append('</engine>')
  output.append('</engines>')
  return '\n'.join(output)


def GetIbusConfigTextProto(param_dict, engines):
  """Outputs a TextProto data for iBus config.

  Args:
    param_dict: A dictionary to embed options into output string.
        For example, {'product_name': 'Mozc'}.
    engines: A dictionary from a property name to a list of property values of
        engines. For example, {'name': ['mozc-jp', 'mozc', 'mozc-dv']}.
  Returns:
    output string in TextProto.
  """
  output = []
  for i in range(len(engines['name'])):
    output.append('engines {')
    for key in engines:
      output.append(GetTextProtoElement(param_dict, key, engines[key][i]))
    output.append('}')
  return '\n'.join(output)


def OutputXml(param_dict, component, engine_common, engines, setup_arg):
  """Outputs a XML data for ibus-daemon.

  Args:
    param_dict: A dictionary to embed options into output string.
        For example, {'product_name': 'Mozc'}.
    component: A dictionary from a property name to a property value of the
        ibus-mozc component. For example, {'name': 'com.google.IBus.Mozc'}.
    engine_common: A dictionary from a property name to a property value that
        are commonly used in all engines. For example, {'language': 'ja'}.
    engines: A dictionary from a property name to a list of property values of
        engines. For example, {'name': ['mozc-jp', 'mozc', 'mozc-dv']}.
    setup_arg: A command line to execute Mozc's configuration tool.
  """
  del engine_common, engines, setup_arg
  print('<component>')
  for key in component:
    print(GetXmlElement(param_dict, key, component[key]))
  print('  <engines exec="%s --xml" />' % param_dict['ibus_mozc_path'])
  print('</component>')

  print('''
<!-- Settings of <engines> and <layout> are stored in ibus_config.textproto -->
<!-- under the user configuration directory, which is either of: -->
<!-- * $XDG_CONFIG_HOME/mozc/ibus_config.textproto -->
<!-- * $HOME/.config/mozc/ibus_config.textproto -->
<!-- * $HOME/.mozc/ibus_config.textproto -->
''')


def OutputCppVariable(param_dict, prefix, variable_name, value):
  print('const char k%s%s[] = "%s";' % (prefix, variable_name.capitalize(),
                                        (value % param_dict)))


def OutputCpp(param_dict, component, engine_common, engines, setup_arg):
  """Outputs a C++ header file for mozc/unix/ibus/main.cc.

  Args:
    param_dict: see OutputXml.
    component: ditto.
    engine_common: ditto.
    engines: ditto.
    setup_arg: ditto.
  """
  guard_name = 'MOZC_UNIX_IBUS_MAIN_H_'
  print(CPP_HEADER % (guard_name, guard_name))
  for key in component:
    OutputCppVariable(param_dict, 'Component', key, component[key])
  for key in engine_common:
    OutputCppVariable(param_dict, 'Engine', key, engine_common[key])
  OutputCppVariable(param_dict, 'Engine', 'Setup', ' '.join(setup_arg))
  for key in engines:
    print('const char* kEngine%sArray[] = {' % key.capitalize())
    for i in range(len(engines[key])):
      print('"%s",' % (engines[key][i] % param_dict))
    print('};')
  print('const size_t kEngineArrayLen = %s;' % len(engines['name']))
  print('const char kEnginesXml[] = R"#(', end='')
  print(GetEnginesXml(param_dict, engine_common, engines, setup_arg))
  print(')#";')
  print('const char kIbusConfigTextProto[] = R"#(', end='')
  print(GetIbusConfigTextProto(param_dict, engines))
  print(')#";')
  print(CPP_FOOTER % guard_name)


def main():
  """The main function."""
  parser = optparse.OptionParser(usage='Usage: %prog [options]')
  parser.add_option('--output_cpp', action='store_true',
                    dest='output_cpp', default=False,
                    help='If specified, output a C++ header. Otherwise, output '
                    'XML.')
  parser.add_option('--branding', dest='branding', default=None,
                    help='GoogleJapaneseInput for the official build. '
                    'Otherwise, Mozc.')
  parser.add_option('--ibus_mozc_path', dest='ibus_mozc_path', default='',
                    help='The absolute path of ibus_mozc executable.')
  parser.add_option('--ibus_mozc_icon_path', dest='ibus_mozc_icon_path',
                    default='', help='The absolute path of ibus_mozc icon.')
  parser.add_option('--server_dir', dest='server_dir', default='',
                    help='The absolute directory path to be installed the '
                    'server executable.')
  (options, unused_args) = parser.parse_args()

  setup_arg = []
  setup_arg.append(os.path.join(options.server_dir, 'mozc_tool'))
  setup_arg.append('--mode=config_dialog')

  param_dict = {
      'product_name': PRODUCT_NAMES[options.branding],
      'ibus_mozc_path': options.ibus_mozc_path,
      'ibus_mozc_icon_path': options.ibus_mozc_icon_path,
  }

  engine_common_props = {
      'description': '%(product_name)s (Japanese Input Method)',
      'language': 'ja',
      'icon': '%(ibus_mozc_icon_path)s',
      'rank': '80',
      # Make sure that the property key 'InputMode' matches to the property name
      # specified to |ibus_property_new| in unix/ibus/property_handler.cc
      'icon_prop_key': 'InputMode',
      'symbol': '&#x3042;',
  }

  # DO NOT change the engine name 'mozc-jp'. The names is referenced by
  # unix/ibus/mozc_engine.cc.
  engines_props = {
      'name': ['mozc-jp'],
      'longname': ['%(product_name)s'],
      'layout': ['default'],
  }

  if options.output_cpp:
    OutputCpp(param_dict, IBUS_COMPONENT_PROPS, engine_common_props,
              engines_props, setup_arg)
  else:
    OutputXml(param_dict, IBUS_COMPONENT_PROPS, engine_common_props,
              engines_props, setup_arg)
  return 0

if __name__ == '__main__':
  sys.exit(main())



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import print_function

import optparse
import sys
import six


class Mapping(object):

  def __init__(self, options):
    self._mapname = options.mapname
    self._key_type = options.key_type
    self._result_type = options.result_type
    self._filename = options.filename

  def PrintLine(self, line):
    columns = line.strip().split('\t')
    if len(columns) != 2:
      return
    (key, value) = columns
    mapname = self._mapname
    if key.startswith('Shift '):
      mapname += 'Shift'
      key = key[len('Shift '):len(key)]

    if self._key_type == 'unsigned short' and not key.startswith('kVK_'):
      key = 'kVK_ANSI_' + key

    if self._result_type == 'const char *':
      if six.PY3:
        value = '"%s"' % ''.join(r'\x%02x' % b for b in value.encode('utf-8'))
      else:
        value = '"%s"' % ''.join(['\\x%x' % ord(c) for c in value])
    elif self._result_type == 'KeyEvent::SpecialKey':
      value = 'KeyEvent::' + value
    print('  (*k%s)[%s] = %s;' % (mapname, key, value))

  def PrintHeader(self):
    print("""// Copyright 2009 Google Inc.  All Rights Reserved.
// Author: mukai
//
// This file is automatically generated by
// generate_mapping.py.
// Do not edit directly and do not include this from any file other
// than KeyCodeMap.mm

namespace {
static std::map<%(key_type)s, %(result_type)s> *k%(mapname)s = nullptr;
static std::map<%(key_type)s, %(result_type)s> *k%(mapname)sShift = nullptr;
static once_t kOnceFor%(mapname)s = MOZC_ONCE_INIT;
void Init%(mapname)s() {
  if (k%(mapname)s != nullptr || k%(mapname)sShift != nullptr) {
    return;
  }
  k%(mapname)s = new(std::nothrow)std::map<%(key_type)s, %(result_type)s>;
  if (k%(mapname)s == nullptr) {
    return;
  }
  k%(mapname)sShift = new(std::nothrow)std::map<%(key_type)s, %(result_type)s>;
  if (k%(mapname)sShift == nullptr) {
    delete k%(mapname)s;
    k%(mapname)s = nullptr;
    return;
  }
""" % {'key_type': self._key_type,
       'mapname': self._mapname,
       'result_type': self._result_type})

  def PrintFooter(self):
    print("""}
}  // namespace
""")

  def Print(self):
    self.PrintHeader()
    for line in open(self._filename):
      self.PrintLine(line)
    self.PrintFooter()


def ParseOption():
  """Parse command line options."""
  parser = optparse.OptionParser()
  parser.add_option('--mapname', dest='mapname')
  parser.add_option('--key_type', dest='key_type')
  parser.add_option('--result_type', dest='result_type')
  parser.add_option('--filename', dest='filename')
  (options, unused_args) = parser.parse_args()

  if not options.mapname:
    print('Error: the output map name should be specified.')
    sys.exit(2)
  if not options.key_type:
    options.key_type = 'unsigned short'
  if not options.result_type:
    print('Error: the result type of the output map should be speicified.')
  if not options.filename:
    print('Error: the file name is not specified.')
  return options


def main():
  options = ParseOption()
  Mapping(options).Print()

if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Generates a bootstrapping Win32 resource script with version info.

  % python gen_win32_resource_header.py --output=out.rc \
      --main=main.rc --version_file=version.txt

See mozc_version.py for the detailed information for version.txt.
"""

__author__ = "yukawa"

import logging
import mozc_version
import optparse
import os
import sys


def ParseOptions():
  """Parse command line options.

  Returns:
    An options data.
  """
  parser = optparse.OptionParser()
  parser.add_option('--version_file', dest='version_file')
  parser.add_option('--output', dest='output')
  parser.add_option('--main', dest='main')

  (options, unused_args) = parser.parse_args()
  return options


def GenerateBuildProfile():
  """Generate Win32 resource script.

  Returns:
    Build profile string.
  """
  build_details = []

  return '; '.join(build_details)


def main():
  """The main function."""
  options = ParseOptions()
  if options.version_file is None:
    logging.error('--version_file is not specified.')
    sys.exit(-1)
  if options.output is None:
    logging.error('--output is not specified.')
    sys.exit(-1)
  if options.main is None:
    logging.error('--input is not specified.')
    sys.exit(-1)

  build_details = GenerateBuildProfile()

  if build_details:
    build_details = ('  (%s)' % build_details)

  version = mozc_version.MozcVersion(options.version_file)
  bootstrapper_template = (
      '#define MOZC_RES_VERSION_NUMBER @MAJOR@,@MINOR@,@BUILD@,@REVISION@\n'
      '#define MOZC_RES_VERSION_STRING "@MAJOR@.@MINOR@.@BUILD@.@REVISION@"\n'
      '#define MOZC_RES_SPECIFIC_VERSION_STRING '
      '"@MAJOR@.@MINOR@.@BUILD@.@REVISION@%s"\n'
      '#include "%s"\n') % (build_details, options.main)

  version_definition = version.GetVersionInFormat(bootstrapper_template)

  old_content = ''
  if os.path.exists(options.output):
    # if the target file already exists, need to check the necessity of update.
    old_content = open(options.output).read()

  if version_definition != old_content:
    open(options.output, 'w').write(version_definition)

if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Filter emoji data file by specified conditions and print specified fields.

Usage:
$ python filter.py [conditions...] --print FIELD < emoji_data.tsv > output.tsv

Conditions are a combination of:
* --has FIELD: requies the emoji to have FIELD.
* --not_have FIELD: requires the emoji not to have FIELD.
* --category CATEGORY: requies the emoji to be in CATEGORY.
"""

__author__ = "kyatoh"

import optparse
import sys

from build_tools import code_generator_util


# The order corresponds to the scheme of emoji_data.tsv.
_FIELDS = [
    'unicode_code_point',
    'actual_data',
    'android_code_point',
    'docomo_code_point',
    'softbank_code_point',
    'kddi_code_point',
    'yomi',
    'unicode_name',
    'japanese_name',
    'docomo_name',
    'softbank_name',
    'kddi_name',
    'category_name_index',
]
_FIELD_TO_INDEX = dict((field, index) for index, field in enumerate(_FIELDS))
_CATEGORY_FIELD_INDEX = len(_FIELDS) - 1


def _CreateOptionParser():
  """Create a OptionParser to parse options for this script."""

  description = ('Filters emoji data file by specified conditions and '
                 'print specified fields. '
                 'FIELD options can be specified either by a field name in '
                 'emoji_data.tsv (e.g. unicode_code_point) or a field index '
                 '(0-origin).')
  parser = optparse.OptionParser(description=description)
  parser.add_option(
      '--has', action='append', dest='required_fields', default=[],
      help='specify field that must exist. '
      'Repeat this option to set multiple constraints.',
      metavar='FIELD')
  parser.add_option(
      '--not_have', action='append', dest='forbidden_fields', default=[],
      help='specify field that must NOT exist. '
      'Repeat this option to set multiple constraints.',
      metavar='FIELD')
  parser.add_option('--category', dest='category', default=[],
                    help='specify category that emojis must have',
                    metavar='CATEGORY')
  parser.add_option(
      '--print', action='append', dest='out_fields', default=[],
      help='specify field that will be printed. '
      'Repeat this option to print multiple fields. '
      'Fields will be separated by a tab.',
      metavar='FIELD')
  return parser


def _ToIndex(field):
  assert field in _FIELD_TO_INDEX or field.isdigit(), (
      'FIELD must be a field name or a field index.')
  if field in _FIELD_TO_INDEX:
    return _FIELD_TO_INDEX[field]
  else:
    assert int(field) < len(_FIELDS), 'FIELD index is too large.'
    return int(field)


def _ShouldPrint(entry, options):
  for field in options.required_fields:
    if not entry[_ToIndex(field)]:
      return False
  for field in options.forbidden_fields:
    if entry[_ToIndex(field)]:
      return False
  if options.category:
    if entry[_CATEGORY_FIELD_INDEX].split('-')[0] != options.category:
      return False
  return True


def _Print(entry, options, output_stream):
  indices = [_ToIndex(field) for field in options.out_fields]
  output_stream.write('\t'.join([entry[i] for i in indices]) + '\n')


def main():
  options = _CreateOptionParser().parse_args()[0]
  stream = code_generator_util.SkipLineComment(sys.stdin)
  stream = code_generator_util.ParseColumnStream(stream, delimiter='\t')
  for entry in stream:
    assert len(entry) == len(_FIELDS), 'Invalid TSV row.'
    if _ShouldPrint(entry, options):
      _Print(entry, options, sys.stdout)


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Copyright 2010-2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Single kanji dictionary generator.

How to run this script:
gen_single_kanji_rewriter_data.py
  --single_kanji_file=single_kanji.tsv
  --variant_file=variant_rule.txt
  --output=single_kanji_data.h
"""

__author__ = "hidehiko"

import codecs
import optparse
import struct

from build_tools import code_generator_util
from build_tools import serialized_string_array_builder


def ReadSingleKanji(stream):
  """Parses single kanji dictionary data from stream."""
  stream = code_generator_util.SkipLineComment(stream)
  stream = code_generator_util.ParseColumnStream(stream, num_column=2)
  outputs = list(stream)
  # For binary search by |key|, sort outputs here.
  outputs.sort(key=lambda x: x[0])

  return outputs


def ReadVariant(stream):
  """Parses variant data from stream."""
  variant_types = []
  variant_items = []

  stream = code_generator_util.SkipLineComment(stream)
  stream = code_generator_util.ParseColumnStream(stream)
  for tokens in stream:
    if len(tokens) == 1:
      variant_types.append(tokens[0])
    elif len(tokens) == 2 and variant_types:
      (target, original) = tokens
      variant_items.append([target, original, len(variant_types) - 1])

  # For binary search by |target|, sort variant items here.
  variant_items.sort(key=lambda x: x[0])

  return (variant_types, variant_items)


def WriteSingleKanji(single_kanji_dic, output_tokens, output_string_array):
  """Writes single kanji list for readings.

  The token output is an array of uint32s, where array[2 * i] and
  array[2 * i + 1] are the indices of key and value in the string array.
  See rewriter/single_kanji_rewriter.cc.
  """
  strings = []
  with open(output_tokens, 'wb') as f:
    for index, (key, value) in enumerate(single_kanji_dic):
      f.write(struct.pack('<I', 2 * index))
      f.write(struct.pack('<I', 2 * index + 1))
      strings.append(key)
      strings.append(value)
  serialized_string_array_builder.SerializeToFile(strings, output_string_array)


def WriteVariantInfo(variant_info,
                     output_variant_types,
                     output_variant_tokens,
                     output_variant_strings):
  """Writes single kanji variants info.

  The token output is an array of uint32s, where array[3 * i],
  array[3 * i + 1] and array[3 * i + 2] are the index of target, index of
  original,  and variant type ID. See rewriter/single_kanji_rewriter.cc.
  """
  (variant_types, variant_items) = variant_info

  serialized_string_array_builder.SerializeToFile(variant_types,
                                                  output_variant_types)

  strings = []
  with open(output_variant_tokens, 'wb') as f:
    for index, (target, original, variant_type) in enumerate(variant_items):
      f.write(struct.pack('<I', 2 * index))
      f.write(struct.pack('<I', 2 * index + 1))
      f.write(struct.pack('<I', variant_type))
      strings.append(target)
      strings.append(original)

  serialized_string_array_builder.SerializeToFile(strings,
                                                  output_variant_strings)


def _ParseOptions():
  parser = optparse.OptionParser()

  parser.add_option('--single_kanji_file', dest='single_kanji_file',
                    help='Single kanji file')
  parser.add_option('--variant_file', dest='variant_file',
                    help='Variant rule file')
  parser.add_option('--output_single_kanji_token',
                    dest='output_single_kanji_token',
                    help='Output Single Kanji token data.')
  parser.add_option('--output_single_kanji_string',
                    dest='output_single_kanji_string',
                    help='Output Single Kanji string data.')
  parser.add_option('--output_variant_types',
                    dest='output_variant_types',
                    help='Output variant types.')
  parser.add_option('--output_variant_tokens',
                    dest='output_variant_tokens',
                    help='Output variant tokens.')
  parser.add_option('--output_variant_strings',
                    dest='output_variant_strings',
                    help='Output variant strings.')

  return parser.parse_args()[0]


def main():
  options = _ParseOptions()

  with codecs.open(options.single_kanji_file, 'r',
                   encoding='utf-8') as single_kanji_stream:
    single_kanji = ReadSingleKanji(single_kanji_stream)

  with codecs.open(options.variant_file, 'r',
                   encoding='utf-8') as variant_stream:
    variant_info = ReadVariant(variant_stream)

  WriteSingleKanji(single_kanji,
                   options.output_single_kanji_token,
                   options.output_single_kanji_string)
  WriteVariantInfo(variant_info,
                   options.output_variant_types,
                   options.output_variant_tokens,
                   options.output_variant_strings)


if __name__ == '__main__':
  main()
